package app.insti.adapter;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.squareup.picasso.Picasso;

import java.util.List;

import app.insti.R;
import app.insti.api.model.SubActivity;

public class SubActivitiesAdapter extends RecyclerView.Adapter<SubActivitiesAdapter.SubActivitiesViewHolder>{

    private Context context;
    private List<SubActivity> subActivityList;

    public SubActivitiesAdapter(Context context,List<SubActivity> subActivityList){
        this.context = context;
        this.subActivityList = subActivityList;
    }

    @NonNull
    @Override
    public SubActivitiesViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        LayoutInflater inflater = LayoutInflater.from(context);
        View view = inflater.inflate(R.layout.resource_layout_sub_activities,parent,false);
        return new SubActivitiesViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull SubActivitiesViewHolder holder, int position) {
        SubActivity subActivity = subActivityList.get(position);
        holder.txtViewSubActivityName.setText(subActivity.getSubActivityName());
        holder.txtViewSubActivityShortDescription.setText(subActivity.getSubActivityShortDescription());
        Picasso.get().load(subActivity.getAvatarUrl()).fit().centerCrop().into(holder.imgViewSubActivity);
        holder.linearLayoutSubActivity.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Toast.makeText(context,  subActivity.getSubActivityName() + " clicked", Toast.LENGTH_SHORT).show();
            }
        });
    }

    @Override
    public int getItemCount() {
        return subActivityList.size();
    }

    public class SubActivitiesViewHolder extends RecyclerView.ViewHolder{
        ImageView imgViewSubActivity;
        TextView txtViewSubActivityName;
        TextView txtViewSubActivityShortDescription;
        LinearLayout linearLayoutSubActivity;
        public SubActivitiesViewHolder(@NonNull View itemView) {
            super(itemView);
            imgViewSubActivity = itemView.findViewById(R.id.imgViewSubActivity);
            txtViewSubActivityName = itemView.findViewById(R.id.txtViewSubActivityName);
            txtViewSubActivityShortDescription = itemView.findViewById(R.id.txtViewSubActivityShortDescription);
            linearLayoutSubActivity = itemView.findViewById(R.id.linearLayoutSubActivity);
        }
    }
}